//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  iRealPro plugin
//
//  Copyright (c)2017 Norman Schmidt
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
version: "2.0"
description: qsTr("This plugin (MacOSX only) exports chord symbols to iRealPro. Internet connection required.")
menuPath: "Plugins.iRealPro."

function convert(symbol) {
    //console.log(symbol)
    symbol = symbol.charAt(0).toUpperCase() + symbol.substr(1)
    symbol = symbol.replace('(','')
    symbol = symbol.replace(')','')
    symbol = symbol.replace('dim','o')
    symbol = symbol.replace('-7b5','h')
    symbol = symbol.replace('ø','h')
    symbol = symbol.replace('m','-')
    return symbol
    }

function sleep(milliseconds) {
    var start = new Date().getTime();
    for (var i = 0; i < 1e7; i++)
        if ((new Date().getTime() - start) > milliseconds) break;
    }

onRun: {
    if (typeof curScore === 'undefined')  Qt.quit()

    cmd('transpose-up')  // work around to make chord anotations available
    sleep(500)
    cmd('transpose-down')

    var chart = '', symbol = ['','','','']
    var firstBar = true, barPosition = 0, position = 0, time , temp

    for (var track = 0; track < curScore.ntracks; track++) {
        var segment = curScore.firstSegment()
        while (segment) {
            var count = 0
            var annotation = segment.annotations[count]

            while (annotation) {
                //console.log(annotation._name())
                if (annotation.type === Element.HARMONY) {
                    position = segment.tick / 480
                    var beat = position - barPosition
                    symbol[beat] = convert(annotation.text)
                    }
                annotation = segment.annotations[++count]
                }

            var element = segment.elementAt(track)
            if (element) {
                //console.log(element._name())
                if (element.type === Element.TIMESIG) {
                    console.log(element.numerator)
                    time = element.numerator
                    }
                if (element.type === Element.BAR_LINE) {
                    if (symbol[0]=='' && symbol[1]=='' && symbol[2]=='' && symbol[3]=='')  //   "       "
                        if (!firstBar)  chart = chart + ' x  |'
                    if (symbol[0]=='' && symbol[1]=='' && symbol[2]=='' && symbol[3]!='')  //   "      X"
                        chart = chart + " " + symbol[3] + ',|'
                    if (symbol[0]=='' && symbol[1]=='' && symbol[2]!='' && symbol[3]=='')  //   "    X  "
                        chart = chart + "  " + symbol[2] + ' |'
                    if (symbol[0]=='' && symbol[1]=='' && symbol[2]!='' && symbol[3]!='')  //   "    X X"
                        chart = chart + " " + symbol[2] + "," + symbol[3] + '|'
                    if (symbol[0]=='' && symbol[1]!='' && symbol[2]=='' && symbol[3]=='')  //   "  X    "
                        chart = chart + "," + symbol[1] + ' |'
                    if (symbol[0]=='' && symbol[1]!='' && symbol[2]=='' && symbol[3]!='')  //   "  X   X"
                        chart = chart + " " + symbol[1] + " " + symbol[3] + '|'
                    if (symbol[0]=='' && symbol[1]!='' && symbol[2]!='' && symbol[3]=='')  //   "  X X  "
                        chart = chart + " " + symbol[1] + "," + symbol[2] + ' |'
                    if (symbol[0]=='' && symbol[1]!='' && symbol[2]!='' && symbol[3]!='')  //   "  X X X"
                        chart = chart + " " + symbol[1] + "," + symbol[2] + "," + symbol[3] + '|'
                    if (symbol[0]!='' && symbol[1]=='' && symbol[2]=='' && symbol[3]=='')  //   "X      "
                        chart = chart + symbol[0] + '   |'
                    if (symbol[0]!='' && symbol[1]=='' && symbol[2]=='' && symbol[3]!='')  //   "X     X"
                        chart = chart + symbol[0] + "   " + symbol[3] + '|'
                    if (symbol[0]!='' && symbol[1]=='' && symbol[2]!='' && symbol[3]=='')  //   "X   X  "
                        chart = chart + symbol[0] + " " + symbol[2] + ' |'
                    if (symbol[0]!='' && symbol[1]=='' && symbol[2]!='' && symbol[3]!='')  //   "X   X X"
                        chart = chart + symbol[0] + " " + symbol[2] + "," + symbol[3] + '|'
                    if (symbol[0]!='' && symbol[1]!='' && symbol[2]=='' && symbol[3]=='')  //   "X X    "
                        chart = chart + symbol[0] + "," + symbol[1] + '  |'
                    if (symbol[0]!='' && symbol[1]!='' && symbol[2]=='' && symbol[3]!='')  //   "X X   X"
                        chart = chart +  symbol[0] + "," + symbol[1] + "  " + symbol[3] + '|'
                    if (symbol[0]!='' && symbol[1]!='' && symbol[2]!='' && symbol[3]=='')   //  "X X X  "
                        chart = chart + symbol[0] + "," + symbol[1] + "," + symbol[2] + ' |'
                    if (symbol[0]!='' && symbol[1]!='' && symbol[2]!='' && symbol[3]!='')  //   "X X X X"
                        chart = chart + symbol[0] + ',' +symbol[1] + ','  +symbol[2] + ',' +symbol[3] + '|'
                    barPosition =  segment.tick / 480
                    if (firstBar)  firstBar = false
                    else  position += time
                    symbol = ['','','','']
                    }
                }
            segment = segment.next
            }
        }

    var title = curScore.title
    var composer = curScore.composer
    var offset = title.indexOf(' by ')
    if (offset > 0) {
        temp = title
        title = temp.substring(0,offset)
        composer = temp.substring(offset+4)
        }
    temp = composer.split(' ')
    if (temp.length == 2) {
        composer = temp[1] + ' ' + temp[0]
        }
    var signature = curScore.keysig + 7
    var  key = ['Cb','Gb','Db','Ab','Eb','Bb','F','C','G','D','A','E','B','F#']
    var header = 'irealbook://-' + title + '=' + composer + '=Medium Swing='
    header = header + key[signature] + '=n=[T' + time + '4'
    chart = chart.substring(0, chart.length - 1) + 'Z'
    var uri = encodeURIComponent(header + chart)
    Qt.openUrlExternally("http://www.normanschmidt.net/irealpro.php?chart=" + uri)
    cmd('undo')  // put score back to the way it was bevore transpose workaround
    sleep(500)
    cmd('undo')
    console.log(header + chart) // print chord chart for debugging
    Qt.quit()
    }
}
